package flightsim.logbook;

import java.io.*;
import java.util.*;
//import java.nio.channels.FileChannel.*;

import javax.swing.filechooser.FileSystemView;




/**
 * Logbook: is a container for entries of the logbook file
 * @author lc
 * $Id: LogBook.java,v 1.2 2007-02-03 12:49:31 lc Exp $
 */
public class LogBook implements Iterable<LogEntry> {

	private List<LogEntry> logEntries = new ArrayList<LogEntry>();
	private File origin;
	
	/**
	 * Read a logbook from a file
	 * @param file
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public LogBook(String file) throws FileNotFoundException, IOException {
		this(new File(file));
	}
	
	/**
	 * Read a logbook from a file
	 * @param file
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public LogBook(File f) throws FileNotFoundException, IOException {
		this.origin = f;
		LERandomAccessFile bb = new LERandomAccessFile(f, "r"); //$NON-NLS-1$
		
		while (bb.getFilePointer() < bb.length()) {
			// read entry hdr
			long startpos = bb.getFilePointer();
			
			byte type = bb.readByte();
			int len = (bb.readByte() & 0xff);
			
			if (len == 0) {
				// FIX 20/11/06
				// some fsx logbook seems to have been padded, so just stop here
				break;
			}
			
			// type 05 main logbook entry
			if (type == 0x5) {
				bb.seek(startpos);
				try {
					LogEntry le = new LogEntry(bb);
					logEntries.add(le);
				} catch (IOException ioe) {
					// not formally an IOE error, but can be
					// a bad entry
					bb.seek(startpos + len);
					// for debugging
					ioe.printStackTrace();
				}
			} else {
				// unknown type, jump to the next entry
				bb.seek(startpos + len);
			}
			
		
		}
		bb.close();
	}
	
	/**
	 * Write logbook to file it used for reading
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void write() throws FileNotFoundException, IOException {
		write(origin);
	}
	
	/**
	 * Write logbook to a specific file
	 * @param to
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void write(File to) throws FileNotFoundException, IOException {
		LERandomAccessFile raf = new LERandomAccessFile(to, "rw"); //$NON-NLS-1$
		for (LogEntry le : this) {
			le.writeEntry(raf);
		}
		// truncate file after the last entry in case the new file is 
		// smaller than original
		raf.setLength(raf.getFilePointer());
		raf.close();
	}
	
	/**
	 * Returns an iterator over log entries
	 */
	public Iterator<LogEntry> iterator() {
		return logEntries.iterator();
	}

	/**
	 * Return the list of entries
	 * @return
	 */
	public List<LogEntry> getEntries() {
		return logEntries;
	}


	public boolean add(LogEntry o) {
		return logEntries.add(o);
	}

	/**
	 * Add all entries to the end of this logbook
	 * @param c
	 * @param index position
	 * @return
	 */
	public boolean addAll(int index, Collection<? extends LogEntry> c) {
		return logEntries.addAll(index, c);
	}
	
	public boolean addAll(Collection<? extends LogEntry> c) {
		return logEntries.addAll(c);
	}

	public File getFile() {
		return origin;
	}

	/**
	 * Delegate method. return the number of entries in the logbook
	 * @return
	 */
	public int size() {
		return logEntries.size();
	}

	/**
	 * Create a new entry and add it to the logbook
	 * @return
	 */
	public LogEntry addNewEntry() {
		LogEntry le = new LogEntry();
		logEntries.add(le);
		return le;
	}

	/**
	 * Try to find a valid logbook file in user home dir
	 * @return null if none founds
	 */
	public static File findLogBook() {
		File homeDir = new File(System.getProperty("user.home")); //$NON-NLS-1$
		File lbFile = null;
		if (homeDir != null) {
			lbFile = findLogBook(homeDir);
			if (lbFile != null) return lbFile;
		}
		homeDir = FileSystemView.getFileSystemView().getDefaultDirectory();
		if (homeDir != null) {
			lbFile = findLogBook(homeDir);
			if (lbFile != null) return lbFile;
		}
		
		return null;
	}
	
	private static File findLogBook(File dir) {
		for (File subDir : dir.listFiles()) {
			if (subDir.isDirectory() && (subDir.getName().toLowerCase().contains("flight simulator x"))) { //$NON-NLS-1$
				File logBookFile = new File(subDir, "Logbook.BIN"); //$NON-NLS-1$
				if (logBookFile.exists()) return logBookFile;
			}
		}
		File logBookFile = new File(dir, "Logbook.BIN"); //$NON-NLS-1$
		if (logBookFile.exists()) return logBookFile;
		return null;
	}


	
	/**
	 * A test main
	 * @param args
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static void main(String[] args) throws FileNotFoundException, IOException {
		LogBook logbook = new LogBook(args[0]);
		for (LogEntry le : logbook) {
			System.out.println("From " + le.getFrom() + " to " + le.getTo()); //$NON-NLS-1$ //$NON-NLS-2$
			System.out.println("Date : " + le.getDate()); //$NON-NLS-1$
			System.out.println("Total time " + le.getTotalTime()); //$NON-NLS-1$
			System.out.println("Plane : " + le.getPlaneDesc() + " reg " + le.getPlaneReg()); //$NON-NLS-1$ //$NON-NLS-2$
			System.out.println("Description : " + le.getDescription()); //$NON-NLS-1$
//			for (int i = 0; i < le.numberLandings(); i++)  {
//				System.out.println("On " + le.getLandingAirport(i) + " " + le.getLandingTimes(i));
//			}
			System.out.println("Unkflags: " + le.getAircraftType() + " unks " + Integer.toHexString(le.unkLog01) + " bb " + Integer.toHexString(le.unkDataType02) + " " + le.unkDataType03 + " " + le.unkDataType04 + " " + le.unkDataType05); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
			System.out.println();
		}
//		logbook.write(new File("logbookdump.bin"));
		
	}


	
}
