package flightsim.logbook.gui.exportfilters;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;

import flightsim.logbook.LogBook;
import flightsim.logbook.LogEntry;
import flightsim.logbook.gui.SwingWorker;


public class PrintExporter implements Printable {

	private List<ExportField> lef;
	private LogBook lb;
	private boolean maxset = false;
	private ProgressMonitor pm;
	private PrinterJob pj;
	
	private static final Font HEADER_FONT = new Font("Serif", Font.BOLD, 12); //$NON-NLS-1$
	private static final Font NORMAL_FONT = new Font("Serif", Font.PLAIN, 12); //$NON-NLS-1$
	
	private static final int LINE_MARGIN = 3;
	
	public PrintExporter(List<ExportField> fields, LogBook lb) {
		this.lef = fields;
		this.lb = lb;
		pm = new ProgressMonitor(null, Messages.getString("PrintExporter.0"), "", 0, 100); //$NON-NLS-1$ //$NON-NLS-2$
		pm.setMillisToDecideToPopup(100);
		pm.setMillisToPopup(100);
		pj = PrinterJob.getPrinterJob();
		pj.setPrintable(this);
	}
	

	public int print(Graphics g, PageFormat pageFormat, int pageIndex) throws PrinterException {
	    Graphics2D g2d = (Graphics2D)g;
	    
	    FontMetrics fm = g2d.getFontMetrics(HEADER_FONT);
	    int hdrh = fm.getHeight();
	    fm = g2d.getFontMetrics(NORMAL_FONT);
	    int nrmh = fm.getHeight();
	  
	    if (pm.isCanceled()) {
	    	pj.cancel();
	    	return NO_SUCH_PAGE;
	    }
	    
	    double pageHeight = pageFormat.getImageableHeight();
	    double pageWidth = pageFormat.getImageableWidth();
	    
	    // calculate number of lines per page
	    int lpp = (int) ((pageHeight - 5 - LINE_MARGIN - hdrh - LINE_MARGIN - 5 - 5) /
	    		(LINE_MARGIN + nrmh));
	    
	    if (!maxset) {
	    	maxset = true;
	    	pm.setMaximum((int) (lb.size() / (lpp)) + 1);
	    }
	    
	    if ((pageIndex*lpp) > lb.size()) return NO_SUCH_PAGE;
	    
	    // column width
	    int colwidth = (int)((pageWidth - 5 - 5) / (lef.size()));
	    
	    g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
	    // draw header and border
	    g2d.setColor(Color.black);
	    g2d.drawLine(3, 3, (int) (pageWidth-3), 3);
	    g2d.drawLine(3, 5 + LINE_MARGIN + hdrh + LINE_MARGIN + 3, (int) (pageWidth-3), 5 + LINE_MARGIN + hdrh + LINE_MARGIN + 3);
	    g2d.drawLine(3, (int)(pageHeight - 3), (int) (pageWidth-3), (int)(pageHeight - 3));
	    g2d.drawLine(3, 3, 3, (int)(pageHeight - 3));
	    g2d.drawLine((int)(pageWidth -3), 3, (int)(pageWidth - 3), (int)(pageHeight - 3));
	    
	    // draw header names
	    int pos = 5;
	    fm = g2d.getFontMetrics(HEADER_FONT);
	    g2d.setFont(HEADER_FONT);
	    for (ExportField ef : lef) {
	    	g2d.setClip(pos, 5 + LINE_MARGIN, colwidth - 3, hdrh);
	    	g2d.drawString(ef.getName(), pos, 5 + LINE_MARGIN + hdrh);
	    	pos += colwidth;
	    }
	    
	    // ok draw each line
	    int start = lpp*pageIndex;
	    List<LogEntry> entries = lb.getEntries();
	    fm = g2d.getFontMetrics(NORMAL_FONT);
	    g2d.setFont(NORMAL_FONT);
    	int ypos = 10 + 2*LINE_MARGIN + hdrh;
	    for (int i = start; i < Math.min(start+lpp, entries.size()); i++) {
	    	LogEntry le = entries.get(i);
	    	int xpos = 5;
	    	for (ExportField ef : lef) {
	    		g2d.setClip(xpos, ypos, colwidth - 3, nrmh);
	    		g2d.drawString(ef.format(le), xpos, ypos + nrmh);
	    		xpos += colwidth;
	    	}
	    	ypos += LINE_MARGIN+nrmh;
	    }
	    
	    pm.setProgress(pageIndex);
	    return PAGE_EXISTS;
	}
	

	public void doPrint() {
//		does not seems to work
//		PrintRequestAttributeSet pes = new HashPrintRequestAttributeSet();
//		pes.add(OrientationRequested.LANDSCAPE);
		boolean doPrint = pj.printDialog();
		if (doPrint) {
			SwingWorker sw = new SwingWorker(){

				@Override
				public Object construct() {
					try {
						pj.print();
						return null;
					} catch (PrinterException e) {
						return e;
					}
				}
				
				@Override
				public void finished() {
					pm.close();
					Object o = get();
					if (o != null && o instanceof Exception) {
						JOptionPane.showMessageDialog(null, Messages.getString("PrintExporter.2"), Messages.getString("PrintExporter.3"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
				
				
			};
			sw.start();
		}
		
	}
}
